/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.parser.generator.conflict;

import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import net.sourceforge.chaperon.grammar.Grammar;
import net.sourceforge.chaperon.parser.generator.ItemSet;

/**
 * This class presents a reduce/reduce conflict
 *
 * @author Stephan Michels
 * @version %version%
 */
public class ReduceReduceConflict extends Conflict
{

  private ItemSet _itemset;

  private int     _firstproduction;

  private int     _secondproduction;

  /**
   * Creates a reduce/reduce conflict
   *
   * @param itemset Itemset, in which the conflict occurs
   * @param firstproduction First production
   * @param secondproduction Second production
   */
  public ReduceReduceConflict(ItemSet itemset, int firstproduction,
                              int secondproduction)
  {
    _itemset = itemset;
    _firstproduction = firstproduction;
    _secondproduction = secondproduction;
  }

  /**
   * Returns the type of conflict
   *
   * @return Type of conflict
   */
  public String getType()
  {
    return "REDUCEREDUCE";
  }

  /**
   * Convert this list to a string
   *
   * @return Content of this list
   */
  public String toString()
  {
    return "Reduce/Reduce conflict: state=" + _itemset + " productions="
           + _firstproduction + "," + _secondproduction;
  }
}
